//-----------------------------------------------------------------------------
// File: SWConnectionEndpoint.cpp
//-----------------------------------------------------------------------------
// Project: Kactus2
// Author: Joni-Matti Mtt
// Date: 2.5.2012
//
// Description:
// Common interface for SW connection endpoints.
//-----------------------------------------------------------------------------

#include "SWConnectionEndpoint.h"

#include <IPXACTmodels/kactusExtensions/ComInterface.h>

#include <common/KactusColors.h>

#include <QPen>

//-----------------------------------------------------------------------------
// Function: SWConnectionEndpoint()
//-----------------------------------------------------------------------------
SWConnectionEndpoint::SWConnectionEndpoint(QGraphicsItem* parent, QVector2D const& dir)
    : ConnectionEndpoint(parent)
{
    setDirection(dir);
}

//-----------------------------------------------------------------------------
// Function: ~SWConnectionEndpoint()
//-----------------------------------------------------------------------------
SWConnectionEndpoint::~SWConnectionEndpoint()
{
}

//-----------------------------------------------------------------------------
// Function: SWConnectionEndpoint::updateInterface()
//-----------------------------------------------------------------------------
void SWConnectionEndpoint::updateInterface()
{
    ConnectionEndpoint::updateInterface();

    if (isInvalid())
    {
        setBrush(QBrush(Qt::red));
        return;
    }

    ConnectionEndpoint::EndpointType endPointType = getType();
    if (endPointType == ENDPOINT_TYPE_UNDEFINED)
    {
        setBrush(QBrush(KactusColors::INVALID_INTERFACE));
    }
    else if (endPointType == ENDPOINT_TYPE_API)
    {
        setBrush(QBrush(KactusColors::ENDPOINT_TYPE_API));
    }
    else if (endPointType == ENDPOINT_TYPE_COM)
    {
        // Generate a hexadecimal color based on the transfer type.
        unsigned int color = qHash(getComInterface()->getTransferType()) & 0x00FFFFFF;

        // Extract the color components and make small modifications.
        unsigned char r = (color - 0xDEADC0DE) >> 16;
        unsigned char g = 255 - ((color & 0x0000FF00) >> 8);
        unsigned char b = (color & 0x000000FF);

        setBrush(QBrush(QColor(r, g, b)));
    }
    else
    {
        Q_ASSERT(false);
    }
}

//-----------------------------------------------------------------------------
// Function: SWConnectionEndpoint::isConnectionValid()
//-----------------------------------------------------------------------------
bool SWConnectionEndpoint::isConnectionValid(ConnectionEndpoint const* other) const
{
    if (!ConnectionEndpoint::isConnectionValid(other))
    {
        return false;
    }

    bool fullyDefined = getType() != ENDPOINT_TYPE_UNDEFINED && other->getType() != ENDPOINT_TYPE_UNDEFINED;

    // Check if the types do not match.
    if (fullyDefined && getType() != other->getType())
    {
        return false;
    }

    return true;
}
